/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef __IRCADDR_H
#define __IRCADDR_H

/**
  * Represents the "nickname!ident@host" address of an IRC user.
  */
class irc_address {
private:
	char * source;
	const char * _ident;
	const char * _host;

public:
	/**
	  * Construct irc_address from a string containing a full address or 
	  * just a nickname. 
	  */
	irc_address(const char * str) : source(NULL) {
		set_address(str);
	}
	irc_address(const irc_address &that) : source(NULL) {
		char * out = irc_strdup(that.nick(), that.ident(), that.host());
		chop_and_set(out);
	}
	~irc_address() {
		delete[] source;
	}

	const char * nick() const { 
		return source; 
	}

	const char * ident() const {
		return _ident;
	}

	const char * host() const {
		return _host;
	}

	bool has_host() const {
		return (*_host != '\0');
	}

	bool has_ident() const {
		return (*_ident != '\0');
	}

	void set_address(const char *);
	void set_nick(const char *);
	void set_ident(const char *);
	void set_host(const char *);

private:
	void chop_and_set(char *);
	static char * irc_strdup(const char *, const char *, const char *);

private:
	irc_address& operator = (const irc_address&);
};
#endif
