/**
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * logging/filelib_logfile.h
 * (c) 2007 Murat Deligonul
 **/
 
#ifndef __LOGGING_FILELIB_LOGFILE_H
#define __LOGGING_FILELIB_LOGFILE_H

#include <string>
#include "logging/ostream_logger.h"
#include "logging/filtered_logger.h"
#include "util/exception.h"
#include "io/filter_types.h"
#include "file_entry.h"

// fwd. declarations
class flib_key;
class file_entry;

namespace logging {

/**
 * A logfile that exists within the ezbounce file library.
 */
class filelib_logfile : public filtered_logger<file_entry::writer_t>, public ostream_logger {
private:
	using filtered_logger<file_entry::writer_t>::chain_t;	

	file_entry * 	 	entry;

public:
	/**
	 * Normal constructor: set all parameters now, ready to log upon 
	 * completion.  If the constructor succeeds, it takes over the file_entry
	 * resource and automatically closes it (and its writer) upon destruction.
	 *
	 * If the constructor fails, an exception is thrown, with the file_entry resource
	 * being returned to the state it was received in.
	 */
	filelib_logfile(const handle_t& handle, file_entry * e, const io::filter_list& filters) :
		logger(handle), 
		filtered_logger<chain_t>(handle), ostream_logger(handle),
		entry(NULL) {
			set_entry(e, filters);
		}

	virtual ~filelib_logfile();
	virtual void flush();

protected:
	/**
	 * Special constructor for derived classes: assign entry later
	 * for more controlled construction.
	 */
	explicit filelib_logfile(const handle_t& handle) :
		logger(handle),
		filtered_logger<chain_t>(handle), ostream_logger(handle),
		entry(NULL) { }

	void  set_entry(file_entry *, const io::filter_list&);
	
	virtual void complete(chain_t *);

	static int sanity_check_filters(const io::filter_list&, std::string&);
	static int validate_directory(const std::string&, flib_key *, std::string&);
	static file_entry * create_file_entry(const std::string&, flib_key *, 
						const io::filter_list&, bool, std::string&);
private:
	filelib_logfile(const filelib_logfile&);
	filelib_logfile& operator=(const filelib_logfile&);
};

} /* namespace logging */
#endif	/* LOGGING_FILELIB_LOGFILE_H */

