/**
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 */
#include "autoconf.h"

#include <exception>
#include <cstring>
#include <cstdio>
#include "ircaddr.h"
#include "util/strings.h"
#include "debug.h"

using util::strings::my_strdup;

/**
  * Set all address fields from a "nick!ident@host" string.
  */ 
void irc_address::set_address(const char * str) 
{
	char * new_str = my_strdup(str);
	chop_and_set(new_str);
}

/**
 * Does the dirty work.
 */
void irc_address::chop_and_set(char * str)
{
	assert(str[0] != ':');
	/* Work backward -- get the host first. */	
	char * t = strrchr(str, '@');
	if (t != NULL) {
		*t = 0;
		_host = t+1;
		
		/* Now find the ident */
		t = strchr(str, '!');
		if (t != NULL) {
			*t = 0;
			_ident = t+1;
		}
		else {
			// XXX: illegal condition; what to do ?
			_ident = "";
		}
	} 
	else {
		/* Ident and host not found. */
		_ident = _host = "";
	}
	delete[] source;
	source = str;
}

char * irc_address::irc_strdup(const char * n, const char * i, const char * h)
{
	int new_len = strlen(n) + strlen(i) + strlen(h) + 2;
	char * out = new char[new_len + 1];
	sprintf(out, "%s!%s@%s", n, i, h);
	return out;
}

/**
  * Set the nickname manually.
  */
void irc_address::set_nick(const char * str) 
{
	chop_and_set( irc_strdup(str, ident(), host()) );
}

/**
  * Set the ident field manually.
  */
void irc_address::set_ident(const char * str) 
{
	chop_and_set( irc_strdup(nick(), str, host()) );
}

/**
  * Set the hostname manually.
  */
void irc_address::set_host(const char * str)
{
	chop_and_set( irc_strdup(nick(), ident(), str) );
}

